@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 import logging
-from typing import List, Union
+from typing import TYPE_CHECKING, List, Union
 
 import numpy as np
 
@@ -15,6 +15,9 @@ from ada.sections.utils import get_section
 from .curves import ArcSegment
 from .points import Node
 
+if TYPE_CHECKING:
+    from ada import Section
+
 
 class Pipe(BackendGeom):
     def __init__(
@@ -166,22 +169,21 @@ class Pipe(BackendGeom):
         return roundoff(d + corr_t / 2.0)
 
     @property
-    def section(self):
-        """
-
-        :return:
-        :rtype: Section
-        """
+    def section(self) -> Section:
         return self._section
 
     @property
-    def n1(self):
+    def n1(self) -> Node:
         return self._n1
 
     @property
-    def n2(self):
+    def n2(self) -> Node:
         return self._n2
 
+    @property
+    def nodes(self) -> list[Node]:
+        return [self.n1, self.n2]
+
     @property
     def units(self):
         return self._units
@@ -255,9 +257,12 @@ class PipeSegStraight(BackendGeom):
     @property
     def solid(self):
         from ada.fem.shapes import ElemType
-        from ada.occ.utils import sweep_pipe
+        from ada.occ.utils import apply_penetrations, sweep_pipe
 
-        return sweep_pipe(self.line, self.xvec1, self.section.r, self.section.wt, ElemType.SOLID)
+        raw_geom = sweep_pipe(self.line, self.xvec1, self.section.r, self.section.wt, ElemType.SOLID)
+
+        geom = apply_penetrations(raw_geom, self.penetrations)
+        return geom
 
     def _generate_ifc_elem(self):
         from ada.ifc.write.write_pipe import write_pipe_straight_seg
@@ -342,7 +347,7 @@ class PipeSegElbow(BackendGeom):
     @property
     def solid(self):
         from ada.fem.shapes import ElemType
-        from ada.occ.utils import sweep_pipe
+        from ada.occ.utils import apply_penetrations, sweep_pipe
 
         i = self.parent.segments.index(self)
         if i != 0:
@@ -350,8 +355,10 @@ class PipeSegElbow(BackendGeom):
             xvec = pseg.xvec1
         else:
             xvec = self.xvec1
+        raw_geom = sweep_pipe(self.line, xvec, self.section.r, self.section.wt, ElemType.SOLID)
 
-        return sweep_pipe(self.line, xvec, self.section.r, self.section.wt, ElemType.SOLID)
+        geom = apply_penetrations(raw_geom, self.penetrations)
+        return geom
 
     @property
     def arc_seg(self) -> ArcSegment:
