@@ -1,11 +1,15 @@
 from __future__ import annotations
 
-from dataclasses import dataclass
-from typing import Iterable, Union
+from dataclasses import dataclass, field
+from typing import TYPE_CHECKING, Iterable, List, Union
 
 import numpy as np
 from pyquaternion import Quaternion
 
+if TYPE_CHECKING:
+    from ada import Part
+    from ada.base.physical_objects import BackendGeom
+
 
 @dataclass
 class Transform:
@@ -28,6 +32,13 @@ class Rotation:
         my_quaternion = Quaternion(axis=self.vector, degrees=self.angle)
         return my_quaternion.rotation_matrix
 
+    def rotate_point(self, p: Union[tuple, list]):
+        p1 = np.array(self.origin)
+        rot_mat = self.to_rot_matrix()
+        p_norm = np.array(p) - p1
+        res = p1 + p_norm @ rot_mat.T
+        return res
+
 
 @dataclass
 class Placement:
@@ -35,6 +46,7 @@ class Placement:
     xdir: Union[list, tuple, np.ndarray] = None
     ydir: Union[list, tuple, np.ndarray] = None
     zdir: Union[list, tuple, np.ndarray] = None
+    scale: float = 1.0
     parent = None
 
     def __post_init__(self):
@@ -70,6 +82,15 @@ class Placement:
             # TODO: Add support for combining rotations as well
         return current_location
 
+    def to_vector_geom(self, **kwargs) -> "Part":
+        from ada.occ.utils import make_ori_vector
+
+        return make_ori_vector("VecGeom", self.origin, self.csys, **kwargs)
+
+    @property
+    def csys(self):
+        return [self.xdir, self.ydir, self.zdir]
+
     def __eq__(self, other: Placement):
         from ada.core.vector_utils import vector_length
 
@@ -78,3 +99,24 @@ class Placement:
                 return False
 
         return True
+
+
+@dataclass
+class Instance:
+    instance_ref: Union["Part", "BackendGeom"]
+    placements: List[Placement] = field(default_factory=list)
+
+    def to_list_of_custom_json_matrices(self):
+        from pyquaternion import Quaternion
+
+        from ada.ifc.utils import create_guid
+
+        matrices = [[self.instance_ref.guid, 0, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1]]
+        for place in self.placements:
+            q1 = Quaternion(matrix=np.array(place.csys))
+            rmat = q1.rotation_matrix
+            matrices.append(
+                [create_guid(), *place.origin.astype(float).tolist(), *np.concatenate(rmat).astype(float).tolist()]
+            )
+
+        return matrices
