@@ -1,9 +1,15 @@
+from typing import List
+
 import numpy as np
 
+from ada.config import Settings
 
-def linear_2dtransform_rotate(origin, point, degrees):
+from .exceptions import VectorNormalizeError
+
+
+def linear_2dtransform_rotate(origin, point, degrees) -> np.ndarray:
     """
-    Rotate
+    Rotate a 2d point given an origin and a degree.
 
     :param origin: (x, y) coordinate of point of rotation
     :param point: (x, y) coordinate of point to rotate
@@ -229,37 +235,75 @@ def sort_points_by_dist(p, points):
     return sorted(points, key=lambda x: vector_length(x - p))
 
 
-def is_point_on_line(a, b, p):
-    ap = p - a
-    ab = b - a
-    result = a + np.dot(ap, ab) / np.dot(ab, ab) * ab
+def is_in_interval(value: float, interval_start: float, interval_end: float, incl_interval_ends: bool = False) -> bool:
+    if incl_interval_ends:
+        return interval_start <= value <= interval_end
+    else:
+        return interval_start < value < interval_end
+
+
+def is_between_endpoints(p: np.ndarray, start: np.ndarray, end: np.ndarray, incl_endpoints: bool = False) -> bool:
+    """Returns if point p is on the line between the points start and end"""
+    if is_null_vector(p, start) or is_null_vector(p, end):
+        if incl_endpoints:
+            return True
+        return False
+
+    ab = end - start
+    ap = p - start
+
+    vec_fraction = get_vec_fraction(ap, ab)
+    on_line_segment = is_in_interval(vec_fraction, 0.0, 1.0, incl_interval_ends=incl_endpoints)
+    return is_parallel(ab, ap) and on_line_segment
+
+
+def get_vec_fraction(vec: np.ndarray, reference_vec: np.ndarray) -> float:
+    """Returns the fraction of the projection of vec onto reference_vec."""
+    return np.dot(vec, reference_vec) / np.dot(reference_vec, reference_vec)
+
+
+def point_on_line(start: np.ndarray, end: np.ndarray, point: np.ndarray) -> np.ndarray:
+    """
+
+    :param start: Start of line
+    :param end: End of line
+    :param point: Point
+    :return:
+    """
+    ap = point - start
+    ab = end - start
+    result = start + get_vec_fraction(ap, ab) * ab
     return result
 
 
-def is_parallel(ab: np.array, cd: np.array, tol=0.0001) -> bool:
+def is_null_vector(ab: np.array, cd: np.array, decimals=Settings.precision) -> bool:
+    """Check if difference in vectors AB and CD is null vector"""
+    return np.array_equal((cd - ab).round(decimals), np.zeros_like(ab))
+
+
+def is_parallel(ab: np.array, cd: np.array, tol=Settings.point_tol) -> bool:
     """Check if vectors AB and CD are parallel"""
-    return True if np.abs(np.sin(angle_between(ab, cd))) < tol else False
+    return float(np.abs(np.sin(angle_between(ab, cd)))) < tol
 
 
-def intersect_calc(A, C, AB, CD):
-    """
-    Function for evaluating an intersection point between two vector-lines (AB & CD).  The function returns
-    variables s & t denoting the scalar value multiplied with the two vector equations A + s*AB = C + t*CD.
+def is_perpendicular(ab: np.array, cd: np.array, tol=Settings.point_tol) -> bool:
+    """Returns if the vectors are perpendicular"""
+    return float(np.abs(np.dot(ab, cd))) < tol
 
-    :param A:
-    :type A:
-    :param C:
-    :type C:
-    :param AB:
-    :type AB:
-    :param CD:
-    :type CD:
-    """
+
+def is_angled(vector_1: np.ndarray, vector_2: np.ndarray) -> bool:
+    """Returns true if 2 vectors is not perpendicular nor parallel to each other"""
+    return not (is_perpendicular(vector_1, vector_2) or is_parallel(vector_1, vector_2))
+
+
+def intersect_calc(a: np.ndarray, c: np.ndarray, ab: np.ndarray, cd: np.ndarray):
+    """Function for evaluating an intersection point between two vector-lines (AB & CD).  The function returns
+    variables s & t denoting the scalar value multiplied with the two vector equations A + s*AB = C + t*CD."""
     # Setting up the equation for use in linalg.lstsq
-    a = np.array((AB, -CD)).T
-    b = C - A
+    matrix = np.array((ab, -cd)).T
+    vec = c - a
 
-    st = np.linalg.lstsq(a, b, rcond=None)
+    st = np.linalg.lstsq(matrix, vec, rcond=None)
 
     s = st[0][0]
     t = st[0][1]
@@ -273,10 +317,7 @@ def intersection_point(v1, v2):
     :param v2:
     :return:
     """
-    if len(list(v1[0])) == 2:
-        is2d = True
-    else:
-        is2d = False
+    is2d = len(list(v1[0])) == 2
 
     v1 = [np.array(list(v) + [0.0]) for v in list(v1)] if is2d else v1
     v2 = [np.array(list(v) + [0.0]) for v in list(v2)] if is2d else v2
@@ -299,7 +340,7 @@ def normalize(curve):
         return [x / max(abs(curve)) for x in curve]
 
 
-def is_point_inside_bbox(p, bbox, tol=1e-3):
+def is_point_inside_bbox(p, bbox, tol=1e-3) -> bool:
     """
 
     :param p: Point
@@ -367,7 +408,7 @@ def convex_hull(points):
     return [v] + extend(u, v, left) + [u] + extend(v, u, right) + [v]
 
 
-def is_coplanar(x1, y1, z1, x2, y2, z2, x3, y3, z3, x4, y4, z4):
+def is_coplanar(x1, y1, z1, x2, y2, z2, x3, y3, z3, x4, y4, z4) -> bool:
     """
     Python program to check if 4 points in a 3-D plane are Coplanar
     Function to find equation of plane.
@@ -447,13 +488,13 @@ def global_2_local_nodes(csys, origin, nodes):
     return res
 
 
-def local_2_global_nodes(nodes, origin, xdir, normal):
+def local_2_global_points(points, origin, xdir, normal):
     """
-    A method for converting a list of nodes (points) in a 2d coordinate system to global 3d coordinates
+    A method for converting a list of points in a 2d coordinate system to global 3d coordinates
 
     :param normal: Normal to 2d plane
     :param origin: Origin of local coordinate system
-    :param nodes: List of points in 2d coordinate system
+    :param points: List of points in 2d coordinate system
     :param xdir: Local X-direction
     :return:
     """
@@ -463,18 +504,25 @@ def local_2_global_nodes(nodes, origin, xdir, normal):
     if type(origin) is Node:
         origin = origin.p
 
-    if type(nodes[0]) is Node:
-        nodes = [no.p for no in nodes]
+    if type(points[0]) is Node:
+        points = [no.p for no in points]
 
-    nodes = [np.array(n, dtype=np.float64) if len(n) == 3 else np.array(list(n) + [0], dtype=np.float64) for n in nodes]
+    points = [
+        np.array(n, dtype=np.float64) if len(n) == 3 else np.array(list(n) + [0], dtype=np.float64) for n in points
+    ]
     yvec = calc_yvec(xdir, normal)
 
-    rmat = rotation_matrix_csys_rotate([xdir, yvec], [X, Y], inverse=True)
-
-    return [np.array(origin, dtype=np.float64) + np.dot(rmat, n) for n in nodes]
+    return transform3d([xdir, yvec], [X, Y], origin, points)
 
 
-def normal_to_points_in_plane(points):
+def transform3d(csys_1, csys_2, origin, points) -> List[np.ndarray]:
+    """Transform points between coordinate systems"""
+    rmat = rotation_matrix_csys_rotate(csys_1, csys_2, inverse=True)
+
+    return [np.array(origin, dtype=np.float64) + np.dot(rmat, n) for n in points]
+
+
+def normal_to_points_in_plane(points) -> np.ndarray:
     """Get normal to the plane created by a list of points"""
     if len(points) <= 2:
         raise ValueError("Insufficient number of points")
@@ -511,25 +559,22 @@ def normal_to_points_in_plane(points):
     return np.array([x if abs(x) != 0.0 else 0.0 for x in list(unit_vector(n))])
 
 
-def unit_vector(vector: np.ndarray):
+def unit_vector(vector: np.ndarray) -> np.ndarray:
     """Returns the unit vector of a given vector"""
     norm = vector / np.linalg.norm(vector)
     if np.isnan(norm).any():
-        raise ValueError(f'Error trying to normalize vector "{vector}"')
+        raise VectorNormalizeError(f'Error trying to normalize vector "{vector}"')
 
     return norm
 
 
-def is_clockwise(points):
+def is_clockwise(points) -> bool:
     """Return true if order of 2d points are sorted in a clockwise order"""
     psum = 0
     for p1, p2 in zip(points[:-1], points[1:]):
         psum += (p2[0] - p1[0]) * (p2[1] + p1[1])
     psum += (points[-1][0] - points[0][0]) * (points[-1][1] + points[0][1])
-    if psum < 0:
-        return False
-    else:
-        return True
+    return not float(psum) < 0
 
 
 def calc_xvec(y_vec, z_vec):
@@ -578,18 +623,18 @@ def is_on_line(data):
         return None
 
 
-def projection_onto_line(p0, n1, n2) -> np.ndarray:
+def projection_onto_line(point: np.ndarray, start: np.ndarray, end: np.ndarray) -> np.ndarray:
     """
 
-    :param p0: Point outside beam
-    :param n1: Start node of beam
-    :param n2: End node of beam
+    :param point: Point outside line
+    :param start: Start node of line
+    :param end: End node of line
     :return: Projection from n1 to p0 onto line. Returns projected line segment
     """
 
-    v = n2 - n1
-    p = p0 - n1
+    v = end - start
+    p = point - start
     angle = angle_between(v, p)
     t0 = np.linalg.norm(p) * np.cos(angle) * unit_vector(v)
     q = t0 - p
-    return p0 + q
+    return point + q
