@@ -1,15 +1,21 @@
 import logging
+from typing import TYPE_CHECKING
 
 from ada import Assembly, Material
 
+if TYPE_CHECKING:
+    from ..concepts import IfcRef
 
-def read_material(ifc_mat) -> Material:
+
+def read_material(ifc_mat, ifc_ref: "IfcRef", assembly: "Assembly") -> Material:
     from ada.materials.metals import CarbonSteel, Metal
 
-    mat_psets = ifc_mat.HasProperties
-    if len(mat_psets) == 0:
-        logging.warning(f'No material found for "{ifc_mat}"')
-        return Material("DummyMat")
+    mat_psets = ifc_mat.HasProperties if hasattr(ifc_mat, "HasProperties") else None
+
+    if mat_psets is None or len(mat_psets) == 0:
+        logging.info(f'No material properties found for "{ifc_mat}"')
+        return Material(ifc_mat.Name)
+
     props = {}
     for entity in mat_psets[0].Properties:
         if entity.is_a("IfcPropertySingleValue"):
@@ -22,6 +28,7 @@ def read_material(ifc_mat) -> Material:
         v=props.get("PoissonRatio", 0.3),
         alpha=props.get("ThermalExpansionCoefficient", 1.2e-5),
         zeta=props.get("SpecificHeatCapacity", 1.15),
+        units=assembly.units,
     )
 
     if "StrengthGrade" in props:
@@ -29,11 +36,10 @@ def read_material(ifc_mat) -> Material:
     else:
         mat_model = Metal(sig_u=None, **mat_props)
 
-    return Material(name=ifc_mat.Name, mat_model=mat_model)
+    return Material(name=ifc_mat.Name, mat_model=mat_model, ifc_ref=ifc_ref, units=assembly.units)
 
 
-def read_ifc_materials(f, a: Assembly):
+def read_ifc_materials(f, a: Assembly, ifc_ref: "IfcRef"):
     for ifc_mat in f.by_type("IfcMaterial"):
-        mat = a.add_material(read_material(ifc_mat))
-
-        print(mat)
+        mat = a.add_material(read_material(ifc_mat, ifc_ref, a))
+        logging.info(f'Importing material "{mat}"')
