@@ -1,5 +1,7 @@
+from __future__ import annotations
+
 import logging
-from typing import Tuple
+from typing import TYPE_CHECKING, List, Tuple, Union
 
 import ifcopenshell
 import ifcopenshell.geom
@@ -10,10 +12,11 @@ from ifcopenshell.util.unit import get_prefix_multiplier
 import ada.core.constants as ifco
 from ada.concepts.transforms import Transform
 from ada.config import Settings
-from ada.core.utils import Counter, get_list_of_files, roundoff
+from ada.core.file_system import get_list_of_files
+from ada.core.utils import roundoff
 
-name_gen = Counter(1, "IfcEl")
-tol_map = dict(m=Settings.mtol, mm=Settings.mmtol)
+if TYPE_CHECKING:
+    from ada import Assembly, Beam
 
 
 def ifc_dir(f: ifcopenshell.file, vec: Tuple[float, float, float]):
@@ -21,12 +24,46 @@ def ifc_dir(f: ifcopenshell.file, vec: Tuple[float, float, float]):
 
 
 def get_tolerance(units):
+    tol_map = dict(m=Settings.mtol, mm=Settings.mmtol)
     if units not in tol_map.keys():
         raise ValueError(f'Unrecognized unit "{units}"')
     return tol_map[units]
 
 
+def ensure_guid_consistency(a: Assembly, project_prefix):
+    """Function to edit the global IDs of your elements when they are arbitrarily created from upstream data dump"""
+    ensure_uniqueness = dict()
+    for p in a.get_all_parts_in_assembly():
+        p.guid = create_guid(project_prefix + p.name)
+
+        if p.guid in ensure_uniqueness.keys():
+            # p_other = ensure_uniqueness.get(p.guid)
+            # ancestors1 = p.get_ancestors()
+            # ancestors2 = p_other.get_ancestors()
+            raise ValueError(f"GUID Uniqueness not maintained for {p.name}")
+        ensure_uniqueness[p.guid] = p
+        for obj in p.get_all_physical_objects(sub_elements_only=True):
+            obj.guid = create_guid(project_prefix + obj.name)
+
+            if obj.guid in ensure_uniqueness.keys():
+                conflicting_obj = ensure_uniqueness[obj.guid]
+                # Handle special case BIM software outputs elements of same name
+                # but one of them are opaque obstruction vols
+                if conflicting_obj.opacity != 0.0:
+                    conflicting_obj.name = conflicting_obj.name + "_INSU"
+                    conflicting_obj.guid = create_guid(project_prefix + obj.name)
+                    ensure_uniqueness[conflicting_obj.guid] = conflicting_obj
+                elif obj.opacity != 0.0:
+                    obj.name = obj.name + "_INSU"
+                    obj.guid = create_guid(project_prefix + obj.name)
+                else:
+                    raise ValueError(f"GUID Uniqueness not maintained for '{obj}' in [{a.name}]")
+
+            ensure_uniqueness[obj.guid] = obj
+
+
 def create_guid(name=None):
+    """Creates a guid from a random name or bytes or generates a random guid"""
     import hashlib
     import uuid
 
@@ -42,18 +79,11 @@ def create_guid(name=None):
     return result
 
 
-def ifc_p(f, p):
-    """
-
-    :param f:
-    :param p:
-    :type f: ifcopenshell.file.file
-    :return:
-    """
+def ifc_p(f: ifcopenshell.file, p):
     return f.create_entity("IfcCartesianPoint", to_real(p))
 
 
-def create_ifc_placement(f, origin=ifco.O, loc_z=ifco.Z, loc_x=ifco.X):
+def create_ifc_placement(f: ifcopenshell.file, origin=ifco.O, loc_z=ifco.Z, loc_x=ifco.X):
     """
     Creates an IfcAxis2Placement3D from Location, Axis and RefDirection specified as Python tuples
 
@@ -61,17 +91,15 @@ def create_ifc_placement(f, origin=ifco.O, loc_z=ifco.Z, loc_x=ifco.X):
     :param origin:
     :param loc_z:
     :param loc_x:
-    :type f: ifcopenshell.file.file
     :return:
     """
 
     ifc_loc_z = f.createIfcDirection(to_real(loc_z))
     ifc_loc_x = f.createIfcDirection(to_real(loc_x))
-    axis2placement = f.createIfcAxis2Placement3D(ifc_p(f, origin), ifc_loc_z, ifc_loc_x)
-    return axis2placement
+    return f.createIfcAxis2Placement3D(ifc_p(f, origin), ifc_loc_z, ifc_loc_x)
 
 
-def create_local_placement(f, origin=ifco.O, loc_z=ifco.Z, loc_x=ifco.X, relative_to=None):
+def create_local_placement(f: ifcopenshell.file, origin=ifco.O, loc_z=ifco.Z, loc_x=ifco.X, relative_to=None):
     """
     Creates an IfcLocalPlacement from Location, Axis and RefDirection,
     specified as Python tuples, and relative placement
@@ -81,7 +109,6 @@ def create_local_placement(f, origin=ifco.O, loc_z=ifco.Z, loc_x=ifco.X, relativ
     :param loc_z:
     :param loc_x:
     :param relative_to:
-    :type f: ifcopenshell.file.file
     :return: IFC local placement
     """
 
@@ -115,13 +142,7 @@ def create_new_ifc_file(file_name, schema):
     return f
 
 
-def assembly_to_ifc_file(a):
-    """
-
-    :param a:
-    :type a: ada.Assembly
-    :return:
-    """
+def assembly_to_ifc_file(a: "Assembly"):
     return generate_tpl_ifc_file(a.name, a.metadata["project"], a.metadata["schema"], a.units, a.user)
 
 
@@ -245,17 +266,7 @@ def create_ifcindexpolyline2d(ifcfile, points2d, seg_index):
 
 
 def create_ifcrevolveareasolid(f, profile, ifcaxis2placement, origin, revolve_axis, revolve_angle):
-    """
-    Creates an IfcExtrudedAreaSolid from a list of points, specified as Python tuples
-
-    :param f:
-    :param profile:
-    :param ifcaxis2placement:
-    :param origin:
-    :param revolve_axis:
-    :param revolve_angle:
-    :return:
-    """
+    """Creates an IfcExtrudedAreaSolid from a list of points, specified as Python tuples"""
     ifcorigin = f.create_entity("IfcCartesianPoint", to_real(origin))
     ifcaxis1dir = f.create_entity(
         "IfcAxis1Placement", ifcorigin, f.create_entity("IfcDirection", to_real(revolve_axis))
@@ -310,8 +321,8 @@ def create_ifcrightcylinder(ifc_file, ifcaxis2placement, height, radius):
     return ifcextrudedareasolid
 
 
-def create_property_set(name, ifc_file, metadata_props):
-    owner_history = ifc_file.by_type("IfcOwnerHistory")[0]
+def create_property_set(name, ifc_file, metadata_props, owner_history):
+
     properties = []
 
     def ifc_value(v_):
@@ -367,16 +378,10 @@ def create_property_set(name, ifc_file, metadata_props):
     return ifc_file.create_entity("IfcPropertySet", **atts)
 
 
-def add_properties_to_elem(name, ifc_file, ifc_elem, elem_props):
-    """
-    :param name:
-    :param ifc_file:
-    :param ifc_elem:
-    :param elem_props:
-    :return:
-    """
-    owner_history = ifc_file.by_type("IfcOwnerHistory")[0]
-    props = create_property_set(name, ifc_file, elem_props)
+def add_properties_to_elem(name, ifc_file, ifc_elem, elem_props, owner_history):
+    logging.info(f'Adding "{name}" properties to IFC Element "{ifc_elem}"')
+
+    props = create_property_set(name, ifc_file, elem_props, owner_history=owner_history)
     ifc_file.createIfcRelDefinesByProperties(
         create_guid(),
         owner_history,
@@ -387,24 +392,21 @@ def add_properties_to_elem(name, ifc_file, ifc_elem, elem_props):
     )
 
 
-def add_multiple_props_to_elem(metadata_props, elem, f):
+def add_multiple_props_to_elem(metadata_props, elem, f, owner_history):
     if len(metadata_props.keys()) > 0:
         if type(list(metadata_props.values())[0]) is dict:
             for pro_id, prop_ in metadata_props.items():
-                add_properties_to_elem(pro_id, f, elem, prop_)
+                add_properties_to_elem(pro_id, f, elem, prop_, owner_history=owner_history)
         else:
-            add_properties_to_elem("Properties", f, elem, metadata_props)
+            add_properties_to_elem("Properties", f, elem, metadata_props, owner_history=owner_history)
 
 
-def to_real(v):
-    """
-
-    :param v:
-    :return:
-    """
+def to_real(v) -> Union[float, List[float]]:
     from ada import Node
 
-    if type(v) is tuple:
+    if type(v) is float:
+        return v
+    elif type(v) is tuple:
         return [float(x) for x in v]
     elif type(v) is list:
         if type(v[0]) is float:
@@ -457,21 +459,32 @@ def add_negative_extrusion(f, origin, loc_z, loc_x, depth, points, parent):
     return opening_element
 
 
-def add_colour(f, ifc_body, name, colour):
-    """
-
-    :param f:
-    :param ifc_body:
-    :param name:
-    :param colour:
-    :return:
-    """
+def add_colour(
+    f,
+    ifc_body: Union[List[ifcopenshell.entity_instance], ifcopenshell.entity_instance],
+    name,
+    colour,
+    transparency=0.0,
+    use_surface_style_rendering=False,
+) -> None:
+    """Add IFcSurfaceStyle using either IfcSurfaceStyleRendering or IfcSurfaceStyleShading"""
+    if colour is None:
+        return None
     colour = f.createIfcColourRgb(name, colour[0], colour[1], colour[2])
-    surfaceStyleShading = f.createIfcSurfaceStyleShading()
-    surfaceStyleShading.SurfaceColour = colour
+
+    if use_surface_style_rendering:
+        surfaceStyleShading = f.createIFCSURFACESTYLERENDERING(colour, transparency)
+    else:
+        surfaceStyleShading = f.createIfcSurfaceStyleShading()
+        surfaceStyleShading.SurfaceColour = colour
+
     surfaceStyle = f.createIfcSurfaceStyle(colour.Name, "BOTH", (surfaceStyleShading,))
     presStyleAssign = f.createIfcPresentationStyleAssignment((surfaceStyle,))
-    f.createIfcStyledItem(ifc_body, (presStyleAssign,), colour.Name)
+    if type(ifc_body) in [list, tuple]:
+        for ifc_b in ifc_body:
+            f.createIfcStyledItem(ifc_b, (presStyleAssign,), colour.Name)
+    else:
+        f.createIfcStyledItem(ifc_body, (presStyleAssign,), colour.Name)
 
 
 def calculate_unit_scale(file):
@@ -485,9 +498,20 @@ def calculate_unit_scale(file):
             unit = unit.ConversionFactor.UnitComponent
         if unit.is_a("IfcSIUnit"):
             unit_scale *= get_prefix_multiplier(unit.Prefix)
+
     return unit_scale
 
 
+def get_unit_type(file):
+    value = calculate_unit_scale(file)
+    if value == 0.001:
+        return "mm"
+    elif value == 1:
+        return "m"
+    else:
+        raise NotImplementedError(f'Unit scale of "{value}" is not yet supported')
+
+
 def scale_ifc_file_object(ifc_file, scale_factor):
     """
     Scale length factor to meter
@@ -531,11 +555,11 @@ def scale_ifc_file_object(ifc_file, scale_factor):
                         return obj_
                     elif obj_.is_a("IfcPressureMeasure") or obj_.is_a("IfcModulusOfElasticityMeasure"):
                         # sf is a length unit.
-                        conv_unit = 1 / sf ** 2
+                        conv_unit = 1 / sf**2
                         obj_.wrappedValue = obj_.wrappedValue * conv_unit
                         return obj_
                     elif obj_.is_a("IfcMassDensityMeasure"):
-                        conv_unit = 1 / sf ** 3
+                        conv_unit = 1 / sf**3
                         obj_.wrappedValue = obj_.wrappedValue * conv_unit
                         return obj_
                     # Unit-less
@@ -561,10 +585,7 @@ def scale_ifc_file_object(ifc_file, scale_factor):
                 old_val = getattr(element, attribute)
                 if old_val is None:
                     continue
-                try:
-                    setattr(element, attribute, scale_all(old_val, scale_factor))
-                except Exception as e:
-                    raise ValueError(e)
+                # setattr(element, attribute, scale_all(old_val, scale_factor))
                 # new_val = getattr(element, attribute)
     return ifc_file
 
@@ -632,7 +653,7 @@ def merge_ifc_files(parent_dir, output_file_name, clean_files=False, include_ele
     print(f"File written in {time.time() - checkpoint:.2f} seconds")
 
 
-def convert_bm_jusl_to_ifc(bm):
+def convert_bm_jusl_to_ifc(bm: "Beam") -> int:
     """
     IfcCardinalPointReference
 
@@ -658,10 +679,6 @@ def convert_bm_jusl_to_ifc(bm):
     19.     top in line with the shear centre
 
     https://standards.buildingsmart.org/IFC/RELEASE/IFC4_1/FINAL/HTML/schema/ifcmaterialresource/lexical/ifccardinalpointreference.htm
-
-    :param bm:
-    :type bm: ada.Beam
-    :return:
     """
     jusl = bm.jusl
     jt = bm.JUSL_TYPES
@@ -696,6 +713,7 @@ def tesselate_shape(shape, schema, tol):
     if serialized_geom is None:
         logging.debug("Starting serialization of geometry")
         serialized_geom = ifcopenshell.geom.tesselate(schema, occ_string, tol)
+
     return serialized_geom
 
 
@@ -720,3 +738,41 @@ def export_transform(f: ifcopenshell.file, transform: Transform):
         ifc_dir(f, X),
     )
     raise NotImplementedError()
+
+
+def get_all_subtypes(entity: ifcopenshell.ifcopenshell_wrapper.entity, subtypes=None):
+    subtypes = [] if subtypes is None else subtypes
+    for subtype in entity.subtypes():
+        if subtype.is_abstract() is False:
+            subtypes.append(subtype.name())
+        get_all_subtypes(subtype, subtypes)
+    return subtypes
+
+
+def get_geom_items(ifc_schema):
+    wrap = ifcopenshell.ifcopenshell_wrapper
+    schema = wrap.schema_by_name(ifc_schema)
+    all_entities = {x.name(): x for x in schema.declarations()}
+    entity: wrap.entity = all_entities["IfcGeometricRepresentationItem"]
+    subtypes = get_all_subtypes(entity)
+
+    return subtypes
+
+
+_geom_items = None
+
+
+def get_representation_items(f: ifcopenshell.file, ifc_elem: ifcopenshell.entity_instance):
+    geom_items = [
+        "IfcTriangulatedFaceSet",
+        "IfcExtrudedAreaSolid",
+        "IfcRevolvedAreaSolid",
+        "IfcClosedShell",
+    ]
+    geom_lower = [i.lower() for i in geom_items]
+    return list(
+        filter(
+            lambda x: hasattr(x, "StyledByItem") and x.is_a().lower() in geom_lower,
+            f.traverse(ifc_elem),
+        )
+    )
