@@ -1,10 +1,15 @@
+from __future__ import annotations
+
 import logging
-from typing import Union
+from typing import TYPE_CHECKING, Union
 
 import numpy as np
 
 from .categories import SectionCat
-from .concept import GeneralProperties, Section
+from .concept import GeneralProperties
+
+if TYPE_CHECKING:
+    from ada.sections.concept import Section
 
 # List of documents the various formulas are based upon
 #
@@ -75,15 +80,15 @@ def calc_box(sec: Section) -> GeneralProperties:
     hb = sec.w_top - sec.t_w
 
     Ix = 4 * (ha * hb) ** 2 / (hb / tb + hb / ty + 2 * ha / ty)
-    Iy = (by * (tb ** 3 + tt ** 3) + 2 * ty * d ** 3) / 12 + e * (h - a) ** 2 + f * (c - h) ** 2 + 2 * g * (b - h) ** 2
+    Iy = (by * (tb**3 + tt**3) + 2 * ty * d**3) / 12 + e * (h - a) ** 2 + f * (c - h) ** 2 + 2 * g * (b - h) ** 2
 
-    Iz = ((sec.t_fbtn + sec.t_ftop) * sec.w_top ** 3 + 2 * d * sec.t_w ** 3) / 12 + (g * hb ** 2) / 2
+    Iz = ((sec.t_fbtn + sec.t_ftop) * sec.w_top**3 + 2 * d * sec.t_w**3) / 12 + (g * hb**2) / 2
     Iyz = 0
     Wxmin = Ix * (hb + ha) / (ha * hb)
     Wymin = Iy / max(sec.h - h, h)
     Wzmin = 2 * Iz / sec.w_top
     Sy = e * (h - a) + ty * (h - tb) ** 2
-    Sz = (sec.t_fbtn + sec.t_ftop) * sec.w_top ** 2 / 8 + g * hb / 2
+    Sz = (sec.t_fbtn + sec.t_ftop) * sec.w_top**2 / 8 + g * hb / 2
     Shary = (Iz / Sz) * 2 * sec.t_w * sfy
     Sharz = (Iy / Sy) * 2 * ty * sfz
     Shceny = 0
@@ -133,19 +138,19 @@ def calc_isec(sec: Section) -> GeneralProperties:
 
     z = (bt * tt * a + hw * ty * b + bb * tb * c) / Ax
 
-    tra = (bt * tb ** 3) / 12 + bt * tt * (hz - tt / 2 - z) ** 2
-    trb = (ty * hw ** 3) / 12 + ty * hw * (tb + hw / 2 - z) ** 2
-    trc = (bb * tb ** 3) / 12 + bb * tb * (tb / 2 - z) ** 2
+    tra = (bt * tb**3) / 12 + bt * tt * (hz - tt / 2 - z) ** 2
+    trb = (ty * hw**3) / 12 + ty * hw * (tb + hw / 2 - z) ** 2
+    trc = (bb * tb**3) / 12 + bb * tb * (tb / 2 - z) ** 2
 
     if tt == ty and tt == tb:
-        Ix = (tt ** 3) * (hw + bt + bb - 1.2 * tt) / 3
+        Ix = (tt**3) * (hw + bt + bb - 1.2 * tt) / 3
         Wxmin = Ix / tt
     else:
-        Ix = 1.3 * (bt * tt ** 3 + hw * ty ** 3 + bb * tb ** 3) / 3
+        Ix = 1.3 * (bt * tt**3 + hw * ty**3 + bb * tb**3) / 3
         Wxmin = Ix / max(tt, ty, tb)
 
     Iy = tra + trb + trc
-    Iz = (tb * bb ** 3 + hw * ty ** 3 + tt * bt ** 3) / 12
+    Iz = (tb * bb**3 + hw * ty**3 + tt * bt**3) / 12
     Iyz = 0
     Wymin = Iy / max(hz - z, z)
     Wzmin = 2 * Iz / max(bb, bt)
@@ -156,11 +161,11 @@ def calc_isec(sec: Section) -> GeneralProperties:
     Sy = Iy / (sec.w_top / 2)
 
     # Sy = (sec.t_w*sec.h/2)(sec.h/2)
-    Sz = (tt * bt ** 2 + tb * bb ** 2 + hw * ty ** 2) / 8
+    Sz = (tt * bt**2 + tb * bb**2 + hw * ty**2) / 8
     Shary = (Iz / Sz) * (tb + tt) * sfy
     Sharz = (Iy / Sy) * ty * sfz
     Shceny = 0
-    Shcenz = ((hz - tt / 2) * tt * bt ** 3 + (tb ** 2) * (bb ** 3) / 2) / (tt * bt ** 3 + tb * bb ** 3) - z
+    Shcenz = ((hz - tt / 2) * tt * bt**3 + (tb**2) * (bb**3) / 2) / (tt * bt**3 + tb * bb**3) - z
     Cy = bb / 2
     Cz = z
 
@@ -218,12 +223,12 @@ def calc_angular(sec: Section) -> GeneralProperties:
 
     # Iz_a + A_a*dcy_a**2
 
-    Iz_a = (1 / 12) * a_h * a_w ** 3 + a_area * a_dcy ** 2
-    Iz_b = (1 / 12) * b_h * b_w ** 3 + b_area * b_dcy ** 2
+    Iz_a = (1 / 12) * a_h * a_w**3 + a_area * a_dcy**2
+    Iz_b = (1 / 12) * b_h * b_w**3 + b_area * b_dcy**2
     Iz = Iz_a + Iz_b
 
-    Iy_a = (1 / 12) * a_w * a_h ** 3 + a_area * a_dcz ** 2
-    Iy_b = (1 / 12) * b_w * b_h ** 3 + b_area * b_dcz ** 2
+    Iy_a = (1 / 12) * a_w * a_h**3 + a_area * a_dcz**2
+    Iy_b = (1 / 12) * b_w * b_h**3 + b_area * b_dcz**2
     Iy = Iy_a + Iy_b
 
     posweb = False
@@ -241,8 +246,8 @@ def calc_angular(sec: Section) -> GeneralProperties:
     b = tz - hw / 2.0
     c = tz / 2.0
     piqrt = np.arctan(1.0)
-    Ax = ty * hw + by * tz + (1 - piqrt) * r ** 2
-    y = (hw * ty ** 2 + tz * by ** 2) / (2 * Ax)
+    Ax = ty * hw + by * tz + (1 - piqrt) * r**2
+    y = (hw * ty**2 + tz * by**2) / (2 * Ax)
     z = (hw * b * ty + tz * by * c) / Ax
     d = 6 * r + 2 * (ty + tz - np.sqrt(4 * r * (2 * r + ty + tz) + 2 * ty * tz))
     e = hw + tz - z
@@ -257,14 +262,14 @@ def calc_angular(sec: Section) -> GeneralProperties:
     else:
         raise ValueError("Currently not implemented this yet")
 
-    Ix = (1 / 3) * (by * tz ** 3 + (hz - tz) * ty ** 3)
-    Iyz = (rl * tz / 2) * (y ** 2 - rj ** 2) - (rk * ty / 2) * (e ** 2 - f ** 2)
+    Ix = (1 / 3) * (by * tz**3 + (hz - tz) * ty**3)
+    Iyz = (rl * tz / 2) * (y**2 - rj**2) - (rk * ty / 2) * (e**2 - f**2)
 
     Wxmin = Ix / d
     Wymin = Iy / max(z, hz - h)
     Wzmin = Iz / max(y, rj)
-    Sy = (ty * e ** 2) / 2
-    Sz = (tz * rj ** 2) / 2
+    Sy = (ty * e**2) / 2
+    Sz = (tz * rj**2) / 2
     Shary = (Iz * tz / Sz) * sfy
     Sharz = (Iy * tz / Sy) * sfz
 
@@ -310,7 +315,7 @@ def calc_tubular(sec: Section) -> GeneralProperties:
 
     dy = sec.r * 2
     di = dy - 2 * t
-    Ax = np.pi * sec.r ** 2 - np.pi * (sec.r - t) ** 2
+    Ax = np.pi * sec.r**2 - np.pi * (sec.r - t) ** 2
     Ix = 0.5 * np.pi * ((dy / 2) ** 4 - (di / 2) ** 4)
     Iy = Ix / 2
     Iz = Iy
@@ -318,7 +323,7 @@ def calc_tubular(sec: Section) -> GeneralProperties:
     Wxmin = 2 * Ix / dy
     Wymin = 2 * Iy / dy
     Wzmin = 2 * Iz / dy
-    Sy = (dy ** 3 - di ** 3) / 12
+    Sy = (dy**3 - di**3) / 12
     Sz = Sy
     Shary = (2 * Iz * t / Sy) * sfy
     Sharz = (2 * Iy * t / Sz) * sfz
@@ -355,11 +360,11 @@ def calc_circular(sec: Section) -> GeneralProperties:
     Sfz = 1.0
     Iyz = 0.0
 
-    Ax = np.pi * sec.r ** 2
-    Iy = (np.pi * sec.r ** 4) / 4
+    Ax = np.pi * sec.r**2
+    Iy = (np.pi * sec.r**4) / 4
     Iz = Iy
-    Ix = 0.5 * np.pi * sec.r ** 4
-    Wymin = 0.25 * np.pi * sec.r ** 3
+    Ix = 0.5 * np.pi * sec.r**4
+    Wymin = 0.25 * np.pi * sec.r**3
     Wzmin = Wymin
 
     Wxmin = Ix / sec.r
@@ -367,7 +372,7 @@ def calc_circular(sec: Section) -> GeneralProperties:
     t = sec.r * 0.99
     dy = sec.r * 2
     di = dy - 2 * t
-    Sy = (dy ** 3 - di ** 3) / 12
+    Sy = (dy**3 - di**3) / 12
     Sz = Sy
     Shary = (2 * Iz * t / Sy) * Sfy
     Sharz = (2 * Iy * t / Sz) * Sfz
@@ -413,33 +418,33 @@ def calc_flatbar(sec: Section) -> GeneralProperties:
     Sfz = 1.0
 
     Ax = w * hz
-    Iy = w * hz ** 3 / 12
-    Iz = hz * w ** 3 / 12
+    Iy = w * hz**3 / 12
+    Iz = hz * w**3 / 12
 
-    bm = 2 * w * hz ** 2 / (hz ** 2 + Ax ** 2)
+    bm = 2 * w * hz**2 / (hz**2 + Ax**2)
     Wymin = Iy / max(h, d)
     Wzmin = 2 * Iz / max(w, w)
     Iyz = 0.0
     if hz == bm:
         ca = 0.141
         cb = 0.208
-        Ix = ca * hz ** 4
-        Wxmin = cb * hz ** 3
+        Ix = ca * hz**4
+        Wxmin = cb * hz**3
     elif hz < bm:
         cn = bm / hz
-        ca = (1 - 0.63 / cn + 0.052 / cn ** 5) * 3
-        cb = ca / (1 - 0.63 / (1 + cn ** 3))
-        Ix = ca * bm * hz ** 3
-        Wxmin = cb * bm * hz ** 2
+        ca = (1 - 0.63 / cn + 0.052 / cn**5) * 3
+        cb = ca / (1 - 0.63 / (1 + cn**3))
+        Ix = ca * bm * hz**3
+        Wxmin = cb * bm * hz**2
     else:
         cn = hz / bm
-        ca = (1 - 0.63 / cn + 0.052 / cn ** 5) * 3
-        cb = ca / (1 - 0.63 / (1 + cn ** 3))
-        Ix = ca * hz * bm ** 3
-        Wxmin = cb * hz * bm ** 3
+        ca = (1 - 0.63 / cn + 0.052 / cn**5) * 3
+        cb = ca / (1 - 0.63 / (1 + cn**3))
+        Ix = ca * hz * bm**3
+        Wxmin = cb * hz * bm**3
 
-    Sy = (w * h ** 2) / 2 + (b - w / 2) * (h ** 2) / 3
-    Sz = hz * ((w ** 2) / 8 + a * (w / 4 + a / 6))
+    Sy = (w * h**2) / 2 + (b - w / 2) * (h**2) / 3
+    Sz = hz * ((w**2) / 8 + a * (w / 4 + a / 6))
 
     Shary = Iz * hz * Sfy / Sz
     Sharz = 2 * Iy * b * Sfz / Sy
@@ -484,21 +489,21 @@ def calc_channel(sec: Section) -> GeneralProperties:
 
     a = hz - 2 * tz
     Ax = 2 * by * tz + a * ty
-    y = (2 * tz * by ** 2 + a * ty ** 2) / (2 * Ax)
-    Iy = (ty * a ** 3) / 12 + 2 * ((by * tz ** 3) / 12 + by * tz * ((a + tz) / 2) ** 2)
+    y = (2 * tz * by**2 + a * ty**2) / (2 * Ax)
+    Iy = (ty * a**3) / 12 + 2 * ((by * tz**3) / 12 + by * tz * ((a + tz) / 2) ** 2)
 
     if tz == ty:
-        Ix = ty ** 3 * (2 * by + a - 2.6 * ty) / 3
+        Ix = ty**3 * (2 * by + a - 2.6 * ty) / 3
         Wxmin = Ix / Iy
     else:
-        Ix = 1.12 * (2 * by * tz ** 3 + a * ty ** 3) / 3
+        Ix = 1.12 * (2 * by * tz**3 + a * ty**3) / 3
         Wxmin = Ix / max(tz, ty)
 
-    Iz = 2 * ((tz * by ** 3) / 12 + tz * by * (by / 2 - y) ** 2) + (a * ty ** 3) / 12 + a * ty * (y - ty / 2) ** 2
+    Iz = 2 * ((tz * by**3) / 12 + tz * by * (by / 2 - y) ** 2) + (a * ty**3) / 12 + a * ty * (y - ty / 2) ** 2
     Iyz = 0
     Wymin = 2 * Iy / hz
     Wzmin = Iz / max(by - y, y)
-    Sy = by * tz * (tz + a) / 2 + (ty * a ** 2) / 8
+    Sy = by * tz * (tz + a) / 2 + (ty * a**2) / 8
     Sz = tz * (by - y) ** 2
 
     Shary = (Iz / Sz) * (2 * tz) * sfy
